/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/*
 * this file contains general debugging functionality we use
 * during development.  None of these functions should be used
 * in a release build, and this file should not be included in
 * release builds.
 */

var gTimeline = null;

function trace_onload() {
    gTimeline = new trace();
}

function trace() {
    // initialize timeline
    this.timelineService = null;
    if ("@mozilla.org;timeline-service;1" in Components.classes) {
        try {
            this.timelineService =
                Components.classes["@mozilla.org;timeline-service;1"].getService(Components.interfaces.nsITimelineService);
            // and check it is actually operatiing (ie, enabled)
            this.timelineService.indent();
            this.timelineService.outdent();
        }
        catch (ex) { this.timelineService = null }
    }
}

trace.prototype = {
    myPerfTimer: {},
    perfLog: 0,
    perfTimer: function(name, title, reset)
    {
        if (!this.perfLog) {
            this.perfLog = getLoggingMgr().getLogger('perf');
            this.perfLog.setLevel(LOG_DEBUG);
        }
        if (typeof reset == 'undefined' || !reset) {
            reset = false;
        }
        var timenow = new Date();
        if (reset || this.myPerfTimer[name] == 0) this.myPerfTimer[name] = timenow;
        var timediff = timenow.getTime()-this.myPerfTimer[name].getTime();
        //dump("[" + name + ":"+title+"] = "+timediff + " ms\n");
        this.perfLog.info("[" + name + ":"+title+"] = "+timediff + " ms");
        this.myPerfTimer[name] = timenow;
    },
    timestamp: function(name, title)
    {
        if (this.timelineService)
            this.timelineService.mark("perfTimer: " + name + ": " + title)
    },

    _repeatStr: function(str, aCount) {
       var res = "";
       while (--aCount >= 0)
         res += str;
       return res;
    },

    dumpDOM: function(node, level, recursive) {
      if (level == undefined) {
        level = 0
      }
      if (recursive == undefined) {
        recursive = true;
      }

      dump(this._repeatStr(" ", 2*level) + "<" + node.nodeName + "\n");
      var i;
      if (node.nodeType == 3) {
          dump(this._repeatStr(" ", (2*level) + 4) + node.nodeValue + "'\n");
      } else {
        if (node.attributes) {
          for (i = 0; i < node.attributes.length; i++) {
            dump(this._repeatStr(" ", (2*level) + 4) + node.attributes[i].nodeName + "='" + node.attributes[i].nodeValue + "'\n");
          }
        }
        if (node.childNodes.length == 0) {
          dump(this._repeatStr(" ", (2*level)) + "/>\n");
        } else if (recursive) {
          dump(this._repeatStr(" ", (2*level)) + ">\n");
          for (i = 0; i < node.childNodes.length; i++) {
            this.dumpDOM(node.childNodes[i], level + 1);
          }
          dump(this._repeatStr(" ", 2*level) + "</" + node.nodeName + ">\n");
        }
      }
    },

    dumpEvent: function(event)
    {
        dump('-EVENT DUMP--------------------------\n');
        dump('type:           '+event.type+'\n');
        dump('eventPhase:     '+event.eventPhase+'\n');
        if ('nodeName' in event.target) {
            dump('target:         '+event.target.nodeName+'\n');
        } else {
            dump('target:         '+event.target+'\n');
        }
        if (event.currentTarget && 'nodeName' in event.currentTarget) {
            dump('currentTarget:  '+event.currentTarget.nodeName+'\n');
        } else {
            dump('currentTarget:  '+event.currentTarget+'\n');
        }
        if (event.originalTarget && 'nodeName' in event.originalTarget) {
            dump('originalTarget: '+event.originalTarget.nodeName+'\n');
        } else {
            dump('originalTarget: '+event.originalTarget+'\n');
        }
        dump('bubbles:        '+event.bubbles+'\n');
        dump('cancelable:     '+event.cancelable+'\n');
        dump('detail:         '+event.detail+'\n');
        dump('button:         '+event.button+'\n');
        dump('isChar:         '+event.isChar+'\n');
        dump("charCode:       "+event.charCode+"\n");
        if (event.charCode) {
            dump("str(charCode):  '"+String.fromCharCode(event.charCode)+"'\n");
        }
        dump('shiftKey:       '+event.shiftKey+'\n');
        dump('ctrlKey:        '+event.ctrlKey+'\n');
        dump('altKey:         '+event.altKey+'\n');
        dump('keyCode:        '+event.keyCode+'\n');
        dump('metaKey:        '+event.metaKey+'\n');
        dump('-------------------------------------\n');
    },
    dumpObject: function(o, name)
    {
        if (typeof(name) == 'undefined') name = 'Object';
        for (var x in o) {
            try {
                dump(name+'[' + x + '] = '+ o[x] + '\n');
            } catch (e) {
                dump(name+'[' + x + '] = <error>\n');
            }
        }
    },
    dumpView: function(view) {
        // Dump some interesting information about the current view.
        dump("\n--------------------------------------\n");
        try {
            if (view) {
                if (view.document) {
                    var doc = view.document;
                    var nle = doc.new_line_endings;
                    var nleName = {0:"EOL_LF", 1:"EOL_CR", 2:"EOL_CRLF", 3:"EOL_MIXED", 4:"EOL_NOEOL"}[nle];
                    dump("view.document.new_line_endings: "+nle+" ("+nleName+")\n");
                } else {
                    dump("view.document is null\n");
                }

                var type = view.getAttribute("type");
                dump("view type: '"+type+"'\n");
                if (type == "editor") {
                    var sciUtilsSvc = Components.classes["@activestate.com/koSciUtils;1"].
                                      getService(Components.interfaces.koISciUtils);
                    var language = view.scintilla.language;
                    dump("language: "+language+"\n");
                    var scimoz = view.scimoz;
                    dump("number of style bits: "+scimoz.styleBits+"\n");
                    var styleMask = (1 << scimoz.styleBits) - 1;
                    dump("style mask: "+styleMask+"\n");
                    dump("current cursor position: "+scimoz.currentPos+"\n");
                    var styleByte = scimoz.getStyleAt(scimoz.currentPos);
                    dump("    style byte: "+styleByte+"\n");
                    var styleNum = styleByte & styleMask;
                    var styleName = sciUtilsSvc.styleNameFromNum(language,
                                                                 styleNum);
                    dump("    lexical style: "+styleNum+" ("+styleName+")\n");
                }

                var scimoz = view.scimoz;
                dump("view.scimoz: "+scimoz+"\n");
                if (scimoz) {
                    dump("view.scimoz.currentPos: "+scimoz.currentPos+"\n");
                    dump("view.scimoz.anchor: "+scimoz.anchor+"\n");
                }
            } else {
                dump("view is null\n");
            }
        } catch(ex) {
            log.exception(ex, "error dumping view");
        }
        dump("--------------------------------------\n");
    },
    getStack: function()
    {

        if (!((typeof Components == "object") &&
              (typeof Components.classes == "object")))
            return "No stack trace available.";

        var frame = Components.stack.caller;
        var str = "<top>";

        while (frame)
        {
            var name = frame.name ? frame.name : "[anonymous]";
            str += "\n" + name + "@" + frame.filename +':' + frame.lineNumber;
            frame = frame.caller;
        }

        return str+"\n";

    },
    /* XXX copied from venkman-utils.js
     * Dumps an object in tree format, recurse specifiec the the number of objects
     * to recurse, compress is a boolean that can uncompress (true) the output
     * format, and level is the number of levels to intitialy indent (only useful
     * internally.)  A sample dumpObjectTree (o, 1) is shown below.
     *
     * + parent (object)
     * + users (object)
     * | + jsbot (object)
     * | + mrjs (object)
     * | + nakkezzzz (object)
     * | *
     * + bans (object)
     * | *
     * + topic (string) 'ircclient.js:59: nothing is not defined'
     * + getUsersLength (function) 9 lines
     * *
     */
    dumpObjectTree: function(o, recurse, compress, level)
    {
        var s = "";
        var pfx = "";

        if (typeof recurse == "undefined")
            recurse = 0;
        if (typeof level == "undefined")
            level = 0;
        if (typeof compress == "undefined")
            compress = true;

        for (var i = 0; i < level; i++)
            pfx += (compress) ? "| " : "|  ";

        var tee = (compress) ? "+ " : "+- ";

        for (i in o)
        {
            var t;
            try
            {
                t = typeof o[i];

                switch (t)
                {
                    case "function":
                        var sfunc = String(o[i]).split("\n");
                        if (sfunc[2] == "    [native code]")
                            sfunc = "[native code]";
                        else
                            sfunc = sfunc.length + " lines";
                        s += pfx + tee + i + " (function) " + sfunc + "\n";
                        break;

                    case "object":
                        s += pfx + tee + i + " (object) " + o[i] + "\n";
                        if (!compress)
                            s += pfx + "|\n";
                        if ((i != "parent") && (recurse))
                            s += this.dumpObjectTree (o[i], recurse - 1,
                                                 compress, level + 1);
                        break;

                    case "string":
                        if (o[i].length > 200)
                            s += pfx + tee + i + " (" + t + ") " +
                                o[i].length + " chars\n";
                        else
                            s += pfx + tee + i + " (" + t + ") '" + o[i] + "'\n";
                        break;

                    default:
                        s += pfx + tee + i + " (" + t + ") " + o[i] + "\n";
                }
            }
            catch (ex)
            {
                s += pfx + tee + i + " (exception) " + ex + "\n";
            }

            if (!compress)
                s += pfx + "|\n";

        }

        s += pfx + "*\n";

        return s;

    },
    startTimer: function(name)
    {
        if (this.timelineService) {
            this.timelineService.startTimer(name);
        }
    },
    stopTimer: function(name)
    {
        if (this.timelineService) {
            try {
                this.timelineService.stopTimer(name);
            } catch (e) {
                log.warn('Error calling timelineService.stopTimer('+ name + ')');
            }
        }
    },
    resetTimer: function(name)
    {
        if (this.timelineService) {
            try {
                this.timelineService.resetTimer(name);
            } catch (e) {
                log.warn('Error calling timelineService.resetTimer('+ name + ')');
            }
        }
    },
    markTimer: function(name, text)
    {
        if (this.timelineService) {
            if (typeof(text) == 'undefined' || text == '') {
                this.timelineService.markTimer(name);
            } else {
                try {
                    this.timelineService.markTimerWithComment(name, text);
                } catch (e) {
                    this.timelineService.markTimerWithComment(name + '-' + text);
                }
            }
        }
    },
    mark: function(text)
    {
        if (this.timelineService) {
            this.timelineService.mark(text);
        }
    },
    indent: function(text)
    {
        if (this.timelineService) {
            this.timelineService.indent(text);
        }
    },
    outdent: function(text)
    {
        if (this.timelineService) {
            this.timelineService.outdent(text);
        }
    },
    enter: function(text)
    {
        if (this.timelineService) {
            this.timelineService.enter(text);
            this.timelineService.startTimer(text);
        }
    },
    leave: function(text)
    {
        if (this.timelineService) {
            this.timelineService.stopTimer(text);
            this.markTimer(text);
            this.timelineService.resetTimer(text);
            this.timelineService.leave(text);
        }
    }
};

