/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

var log = getLoggingMgr().getLogger("debugging-properties");

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var myPrefWindow = null;
var startBox;

var gInvocation = null;
var gInvokeType = 0; // what kind of invocation are we displaying?

var gInvocationPrefs = null;
var gURLPrefs = null;
var gMainWindow= null;
var dialog = null;

function SetupDialog() {
    trace_onload();
    try {
        gMainWindow = windowManager_getMainWindow();
        myPrefWindow = new koPrefWindow('dialog-debugging-properties');
        dialog = document.getElementById('dialog-debugging-properties');

        if (window.arguments.length > 0) {
            gInvocation = window.arguments[0].invocation;
            gInvokeType = window.arguments[0].invoke_type;
            gURLPrefs = window.arguments[0].prefs;
        } else {
            // XXX fix default langauge
            gInvokeType = Components.interfaces.koIInvocation.INVOKE_DEBUG;
            gInvocation = Components.classes["@activestate.com/koInvocation;1?type=Perl"].createInstance();
            gURLPrefs = gMainWindow.getEffectivePrefsForFile();
        }
        gInvocationPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        gInvocationPrefs.id = "Invocations";
        if (gURLPrefs.hasPref("Invocations")) {
            gInvocationPrefs.update(gURLPrefs.getPref('Invocations'));
        } else {
            // no preexisting invocation prefs, set the ones we have now
            gInvocationPrefs.setPref(gInvocation.persistableInstancePreferences.id,
                                     gInvocation.persistableInstancePreferences);
        }

        FillInvocationPopup(gInvocation);
        // add the debugger component to the ui
        var interpparams = document.getElementById('interpparams');
        var el = document.createElementNS(XUL_NS, 'debugOptions');
        el.setAttribute('type', gInvocation.name.toLowerCase())
        interpparams.appendChild(el);

        var propPanel = document.getElementById('generalProperties');
        el = document.createElementNS(XUL_NS, 'debugProperties');
        el.setAttribute('type', gInvocation.name.toLowerCase())
        el.setAttribute('prefwidgetauto', 'true');
        propPanel.appendChild(el);

        window.sizeToContent();
        dialog.moveToAlertPosition();

        UpdateInvocation(gInvocation);

        gMainWindow.gOpeningDialog = false;
    } catch(e) {
        log.exception(e);
    }
}

function UpdateInvocation(invocation) {
    try {
        gInvocation = invocation;
        myPrefWindow.init(gInvocation.currentInstancePreferences);
        myPrefWindow.onsinglepageload();
        toggleCGITabs();
        // for convenience, focus the params field.  "tabindex=" will make this redundant when it works!
        document.getElementById("params").focus();
    } catch(e) {
        log.exception(e);
    }
}

function FillInvocationPopup(currentInvocation) {
    try {
        var menupopup = document.getElementById('invocation-configuration-popup');
        while (menupopup.childNodes.length) {
            menupopup.removeChild(menupopup.childNodes[0]);
        }
        var configurations = new Object();
        gInvocationPrefs.getPrefIds(configurations, new Object());
        //dump('got '+configurations.value.length+' invocations for this file\n');
        if (configurations.value.length <= 1)
            document.getElementById('deleteButton').setAttribute('disabled','true');
        configurations = configurations.value;
        //dump("current invocation name "+currentInvocation.currentInstancePreferences.id+"\n");
        //currentInvocation.currentInstancePreferences.dump(4);
        //gInvocationPrefs.dump(0);
        for (var i=0; i < configurations.length; i++) {
            var invocation_pref = gInvocationPrefs.getPref(configurations[i]);
            var menuitem = document.createElementNS(XUL_NS, 'menuitem');
            menuitem.setAttribute('label', configurations[i]);
            menupopup.appendChild(menuitem);
            //dump("Adding config "+configurations[i]+"\n");
            //dump("       config "+invocation_pref.id+"\n");
            //invocation_pref.dump(4)
            if (invocation_pref.id == currentInvocation.currentInstancePreferences.id) {
                document.getElementById('invocation-configuration').setAttribute('label', configurations[i]);
            }
        }
    } catch(e) {
        log.exception(e);
    }
}

function changePrefs(prefs) {
    /* we need ot start with clean invocation prefs, then update them with the prefs
       that were stored for this invocation configuration */
    var catOb = new Object();
    var instOb = new Object();
    var filename = "";
    var inputfilename = "";
    if (gInvocation.currentInstancePreferences.hasPref("filename"))
        filename = gInvocation.currentInstancePreferences.getStringPref("filename");
    if (gInvocation.currentInstancePreferences.hasPref("inputfile"))
        inputfilename = gInvocation.currentInstancePreferences.getStringPref("inputfile");

    gInvocation.getDefaultPreferences(catOb, instOb);
    gInvocation.currentCategoryPreferences = catOb.value;
    gInvocation.currentInstancePreferences = instOb.value;

    var iprefs = gInvocation.currentInstancePreferences;
    iprefs.update(prefs);
    iprefs.id = prefs.id;
    iprefs.setStringPref("filename", filename);
    iprefs.setStringPref("inputfile", inputfilename);
}

function ChangeInvocation() {
    var name = document.getElementById('invocation-configuration').getAttribute("label");
    var invocation_prefs = gInvocationPrefs.getPref(name);
    invocation_prefs.id = name;
    changePrefs(invocation_prefs);
    UpdateInvocation(gInvocation);
}

function doSaveNewInvocation() {
    // first, get a name for the new invocation
    try {
        var name = dialog_prompt("Enter a new configuration name.  The new "+
                                 "configuration will be based on the currently "+
                                 "selected '"+gInvocation.currentInstancePreferences.id+"' configuration.",
                                 "Configuration Name", "",
                                 "Configuration Name");
        if (!name) return;
        changePrefs(gInvocation.persistableInstancePreferences);
        gInvocation.currentInstancePreferences.id = name;
        gInvocationPrefs.setPref(gInvocation.persistableInstancePreferences.id,
                                 gInvocation.persistableInstancePreferences);

        //gInvocation.currentInstancePreferences.dump(0)

        FillInvocationPopup(gInvocation);
        myPrefWindow.prefset = gInvocation.currentInstancePreferences;
        document.getElementById('deleteButton').removeAttribute('disabled');
    } catch(e) {
        log.exception(e);
    }
}

function doDeleteInvocation() {
    try {
        var name = document.getElementById('invocation-configuration').getAttribute('label');
        var configurations = new Object();
        gInvocationPrefs.getPrefIds(configurations, new Object());
        if (configurations.value.length <= 1) {
            return;
        } else {
            if (dialog_yesNo("Are you sure you want to delete the "+name+" configuration?") != "Yes") return;
        }
        gInvocationPrefs.deletePref(name);
        gInvocationPrefs.getPrefIds(configurations, new Object());
        var invocation_prefs = gInvocationPrefs.getPref(configurations.value[0]);
        invocation_prefs.id = name;
        changePrefs(invocation_prefs);

        FillInvocationPopup(gInvocation);
        UpdateInvocation(gInvocation);
    } catch(e) {
        log.exception(e);
    }
}

function _setReturnVar(name, val) {
  if (window.arguments && window.arguments[0])
    window.arguments[0][name]=val;
  return true;
}

function _setResult(res) {
    return _setReturnVar("res", res)
}

function Ok() {
    try {
        if (!myPrefWindow.onSinglePageOK())
            return false;
        // get the invocation to validate the preference set.
        var bad_pref_id = new Object();
        var message = new Object();
        var ok = gInvocation.validate(gInvokeType, bad_pref_id, message)
        if (!ok) {
            var badWidget = myPrefWindow.findElementWithPreferenceId(bad_pref_id.value);
            if (badWidget)
                badWidget.focus();
            dialog_alert(message.value);
            return false;
        }

        // save our prefs back into the files prefs now
        if (!gURLPrefs.hasPref("Invocations")) {
            gURLPrefs.setPref('Invocations', gInvocationPrefs);
        } else {
            gURLPrefs.getPref('Invocations').update(gInvocationPrefs);
        }

        _setReturnVar("invocation", gInvocation);
        _setResult("ok");
    } catch(e) {
        log.exception(e);
    }
    return true;
}

function Cancel() {
  _setResult("cancel");
  return true;
}

function doClose() {
  doCancel();
  window.close();
}

// Utility functions for the various panels.
function doBrowseForDir(widget_id) {
    if (!widget_id)
        widget_id = "cwd";
    var textField = document.getElementById(widget_id);
    var dir = filepicker_getFolder(textField.value);
    if (dir) textField.value = dir;
}

function doBrowseForLanguageFile(widget_id, language) {
    if (typeof(language) == 'undefined') language = gInvocation.name;
    var textField = document.getElementById(widget_id);
    var path = filepicker_openFile(null, textField.value,
                                   null, // title
                                   language); // default filter name
    if (path != null) {
        textField.value = path;
    }
}

function doBrowseForXMLInputFile(widget_id) {
    if (!widget_id) return;
    var textField = document.getElementById(widget_id)
    var path = filepicker_openFile(null, textField.value,
                                   null, // title
                                   "XML", // default filter name
                                   ["XML", "XSLT", "HTML", "All"]);
    if (path != null) {
        textField.value = path;
    }
}

function doBrowseForCommandFile(widget_id) {
    if (!widget_id) return;
    var textField = document.getElementById(widget_id)
    var path = filepicker_openExeFile(null, textField.value);
    if (path != null) {
        textField.value = path;
    }
}


function toggleCGITabs() {
    try {
        var usecgi = document.getElementById("debuggerRunAsCGI");
        if (!usecgi) return;
        var checked = usecgi.getAttribute("checked");
        var t1 = document.getElementById("tab-cgiEnvironment");
        var t2 = document.getElementById("tab-cgiInput");
        if (checked=="true") {
            t1.removeAttribute('hidden');
            t2.removeAttribute('hidden');
        } else {
            t1.setAttribute('hidden','true');
            t2.setAttribute('hidden','true');
        }
    } catch (e) {
        log.exception(e);
    }
}

function updateInterpreterArgumentsWithDir(menuitem) {
    try {
        var dir = filepicker_getFolder();
        if (!dir) return;
        var val = menuitem.getAttribute('value');
        var params = document.getElementById('executable-params');
        if (params.value) {
            params.value += ' '+val+' '+dir;
        } else {
            params.value = val+' '+dir;
        }
        document.getElementById('executable-params').focus();
    } catch (e) {
        log.exception(e);
    }
}

function updateInterpreterArguments(menuitem) {
    try {
        var val = menuitem.getAttribute('value');
        var params = document.getElementById('executable-params');
        if (params.value) {
            params.value += ' '+val;
        } else {
            params.value = val;
        }
        document.getElementById('executable-params').focus();
    } catch (e) {
        log.exception(e);
    }
}
