/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peGUI();

function peGUI() {
    this.log = getLoggingMgr().getLogger("peGUI");
    try {
        this.GuiBuilder = Components.classes["@activestate.com/koGuiBuilderService;1"].
                  getService(Components.interfaces.koIGuiBuilderService);
    } catch (e) {
        this.log.error(e);
        this.GuiBuilder = null;
    }
    this.name = 'peGUI';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peGUI.prototype.constructor = peGUI;

peGUI.prototype.init = function() {
}

peGUI.prototype.registerCommands = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.registerCommand('cmd_editGUIDialog', this);
        extensionManager.registerCommand('cmd_testGUIDialog', this);
    } catch (e) {
        this.log.error(e);
    }
}
peGUI.prototype.registerEventHandlers = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.addEventHandler(Components.interfaces.koIPart_dialog,'onUpdatePart',this);
        extensionManager.addEventHandler(Components.interfaces.koIPart_dialog,'ondblclick',this);
    } catch (e) {
        this.log.error(e);
    }
}
peGUI.prototype.registerMenus = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,
                                        'Edit Dialog...',
                                        'cmd_editGUIDialog',
                                        null,
                                        null,
                                        true);
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,'Test Dialog...','cmd_testGUIDialog');
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,'Refresh Status','cmd_refreshStatus');
    } catch (e) {
        this.log.error(e);
    }
}

peGUI.prototype.supportsCommand = function(command, item)
{
    if (!item) {
        item = gFocusedProjectView.getSelectedItem();
    }
    switch (command) {
        case 'cmd_editGUIDialog':
        case 'cmd_testGUIDialog':
            return item != null;
        default:
            break;
    }
    return false;
}

peGUI.prototype.isCommandEnabled = function(command, item)
{
    if (!item) {
        item = gFocusedProjectView.getSelectedItem();
    }
    switch (command) {
        case 'cmd_editGUIDialog':
            return item != null;
        case 'cmd_testGUIDialog':
            return item && item.part.hasAttribute('startup')
        default:
            break;
    }
    return false;
}

peGUI.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_editGUIDialog':
        item = gFocusedProjectView.getSelectedItem();
        this.GuiBuilder.Open(item.url, '');
        break;
    case 'cmd_testGUIDialog':
        var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                          getService(Components.interfaces.koISysUtils);
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                      getService(Components.interfaces.koIInfoService);
        item = gFocusedProjectView.getSelectedItem();
        var startupfile = uriparse_URIToLocalPath(
                item.part.getStringAttribute('startup'));
        // XXX Move this logic to language services -- that way it will work with
        // other languages when GUIBuilder adds support for new languages
        var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].
              getService(Components.interfaces.koILanguageRegistryService);
        var language = languageRegistry.suggestLanguageForFile(startupfile)
        var cmd = '';
        var interp = '';
        switch (language) {
            case 'Perl':
                interp = gPrefs.getStringPref("perlDefaultInterpreter");
                if (interp == '') {
                    if (infoSvc.isWindows) {
                        interp = 'wperl';
                    } else {
                        interp = 'perl';
                    }
                    interp = sysUtilsSvc.Which(interp);
                    if (!interp) {
                        alert("To test this Perl dialog you must have " +
                              "Perl on your PATH or the Perl default " +
                              "interpreter preference must be correctly " +
                              "set.");
                        return;
                    }
                }
                // XXX somewhat of a hack -- should not have to do this each time.
                var perlInfoEx = Components.classes["@activestate.com/koPerlInfoEx;1"].createInstance(Components.interfaces.koIPerlInfoEx);
                perlInfoEx.installationPath = perlInfoEx.getInstallationPathFromBinary(interp);
                var modules = new Array();
                modules.push('Tk')
                if (! perlInfoEx.haveModules(modules.length, modules)) {
                    alert("Perl Dialogs need the Tk module to be tested." +
                          " The Perl installation at '" +
                          perlInfoEx.installationPath +
                          "' does not have Tk installed.");
                    return;
                }
                cmd = '"' + interp + '" "' + startupfile + '"';
                break;
            case 'Tcl':
                interp = gPrefs.getStringPref("wishDefaultInterpreter");
                if (interp == '') {
                    interp = 'wish';
                    if (sysUtilsSvc.Which(interp) == '') {
                        alert("Testing this dialog with " + language +
                              " requires that '" + interp +
                              "' be on your PATH or that the " + language +
                              " default interpreter preference be " +
                              "correctly set.");
                        return;
                    }
                }
                cmd = interp + ' "' + startupfile +'"';
                break;
            case 'Python':
                interp = gPrefs.getStringPref("pythonDefaultInterpreter");
                if (interp == '') {
                    if (infoSvc.isWindows) {
                        interp = 'pythonw';
                    } else {
                        interp = 'python';
                    }
                    if (sysUtilsSvc.Which(interp) == '') {
                        alert("Testing this dialog with " + language +
                              " requires that '" + interp +
                              "' be on your PATH or that the " + language +
                              " default interpreter preference be " +
                              "correctly set.");
                        return;
                    }
                }
                cmd = interp + ' "' + startupfile +'"';
                break;
            default:
                alert("This version of Komodo does not know how to " +
                      "test Dialogs with " + language);
                return;
        }
        if (cmd) {
            var userEnvSvc = Components.classes["@activestate.com/koUserEnviron;1"].
                  getService(Components.interfaces.koIUserEnviron);
            var env = userEnvSvc.GetEncodedEnvironment();
            Run_RunCommand(window,
                           cmd,
                           null, // cwd
                           env, // env
                           false, // insertOutput
                           false, // operateOnSelection
                           true, // doNotOpenOutputWindow
                           'no-console', // runIn
                           0, // parseOutput
                           '', // parseRegex
                           0, // showParsedOutputList
                           false); // saveInMRU
        }
        break;
    default:
        break;
    }
}

peGUI.prototype.ondblclick = function(item,event) {
    this.GuiBuilder.Open(item.url, '');
}

peGUI.prototype.onUpdatePart = function(part,event)
{
    part.iconopenstyle['guifile'] = 1
    part.iconstyle['guifile'] = 1
    part.iconopenstyle['projectFolderOpen'] = 0
    part.iconstyle['projectFolderClosed'] = 0
}

function peGUI_addDialog(item)
{
    guiBuilder = Components.classes["@activestate.com/koGuiBuilderService;1"].
                  getService(Components.interfaces.koIGuiBuilderService);
    // get the current selection, then createthe file
    var index = -1;
    var basedir = '';
    if (item && item.part) {
        // Finding the basedir is somewhat broken if an item is selected in the toolbox
        // but it's not a folder -- its base directory will be used.  Harumph.
        var project = item.part.project;
        index = gFocusedProjectView.getIndexByPart(item);
        if (item.url) {
            var url = item.url;
            if (item.type != 'folder') {
                // Projects and files have URLs which are not the directory but the URL of the files!
                url = url.substr(0, url.lastIndexOf('/'))
            }
            basedir = uriparse_URIToLocalPath(url);
        }
    }
    var validName = false;
    var filename, baseName;
    var validNameRe = /^[a-zA-Z_]\w*\.ui$/;
    while (validName == false) {
        filename = filepicker_saveFile(basedir, "Dialog.ui");
        if (filename == null) return; // user canceled
        baseName = uriparse_baseName(filename);
        if (baseName.match(validNameRe) != null) {
            validName = true;
        }
        if (!validName) {
            alert("Invalid name - dialog filenames must start with a " +
                  "letter and contain only letters, numbers and the " +
                  "underscore character (_), and should end in .ui")
        }
    }

    var ask = true;
    if (gPrefs.hasPref('defaultGUILanguage')) {
        var pref = gPrefs.getStringPref('defaultGUILanguage');
        if (pref != '') {
            language = pref;
            ask = false;
        }
    }
    if (ask) {
        var ids = new Object();
        var labels = new Object();
        guiBuilder.getLanguages(new Object(), ids, new Object(), labels);
        var items = new Array();
        for (var i = 0; i < ids.value.length; i++) {
            item = new Object();
            item.id = ids.value[i];
            item.text = labels.value[i];
            items.push(item);
        }

        items = dialog_selectFromList("Choose a language for the new dialog",
                                     "Pick a language to use for this dialog",
                                     items,
                                     'one',
                                     null,
                                     'gui_language');
        if (items == null) {
            // User pressed cancel or escape.
            return;
        }
        language = items[0].id;
    }

    var part = Components.classes["@activestate.com/koPart?type=dialog;1"]
                                    .createInstance(Components.interfaces.koIPart_dialog);
    part.setStringAttribute('url', uriparse_localPathToURI(filename));
    part.setStringAttribute('language', language); // used where? XXX
    gFocusedProjectView.manager.addItem(part,index);
    guiBuilder.Create(part.getStringAttribute('url'), language);
}

function peGUI_editDialog(url) {
    var guiBuilder = Components.classes["@activestate.com/koGuiBuilderService;1"].
                  getService(Components.interfaces.koIGuiBuilderService);
    guiBuilder.Open(url, '');
}