from xpcom import components, nsError, ServerException
from xpcom._xpcom import PROXY_SYNC, PROXY_ALWAYS, PROXY_ASYNC
from koLintResult import *
from koLintResults import koLintResults
import os, sys, re
import tempfile
import string
import process
import koprocessutils
import which

from koLanguageServiceBase import *

import logging
log = logging.getLogger("RubyLanguage")
#log.setLevel(logging.DEBUG)

def registerLanguage(registery):
    registery.registerLanguage(koRubyLanguage())
    
    
class koRubyLanguage(KoLanguageBase):
    name = "Ruby"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{3CFC5F18-0288-47e9-BF2D-061329D00FB6}"

    accessKey = 'r'
    primary = 1
    defaultExtension = ".rb"
    supportsSmartIndent = "python"
    shebangPatterns = [ re.compile(ur'\A#!.*ruby.*$', re.IGNORECASE | re.MULTILINE),]
    
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_RUBY)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 1
        return self._lexer

    commentDelimiterInfo = {
        "line": [ "#" ],
        "block": [ ("=begin", "=end") ],
    }
    _dedenting_statements = [u'yield', u'raise', u'return', u'break', u'next']

    _keywords = ["BEGIN", "class", "ensure", "nil", "self", "when",
                 "END", "def", "false", "not", "super", "while",
                 "alias", "defined", "for", "or", "then", "yield",
                 "and", "do", "if", "redo", "true",
                 "begin", "else", "in", "rescue", "undef",
                 "break", "elsif", "module", "retry", "unless",
                 "case", "end", "next", "return", "until"]

    def get_interpreter(self):
        if self._interpreter is None:
            self._interpreter = components.classes["@activestate.com/koRubyInfoEx;1"].getService()
        return self._interpreter
    
    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koRubyLinter;1")
        return self._linter

#---- components
class KoRubyInfoEx:
    _com_interfaces_ = [components.interfaces.koIAppInfoEx]
    _reg_clsid_ = "{7f6d9dbd-1ee3-4827-9838-acf34d2a47e3}"
    _reg_contractid_ = "@activestate.com/koRubyInfoEx;1"
    _reg_desc_ = "Ruby Information"

    def __init__(self):
        # The specific installation path for which the other attributes
        # apply to. It may be required for this to be set for certain
        # attributes to be determined.
        # attribute wstring installationPath;
        self.installationPath = ''

        # The path to the executable for the interpreter e.g. "usr/bin/perl"
        # attribute wstring executablePath;
        self.executablePath = ''
        self._executables = []

        # True if user is licensed for the current version of this app.
        # (For an app that does not require a license this will always be
        # true.)
        # attribute boolean haveLicense;
        self.haveLicense = 1
        
        # The build number. (May be null if it is not applicable.)
        # attribute long buildNumber;
        self.buildNumber = 0

        # version (duh, null if don't know how to determine)
        # attribute string version;
        version = ''
        
        # path to local help file (null if none)
        # attribute wstring localHelpFile;
        self.localHelpFile = ''

        # Web URL to main help file (null if none)
        # attribute wstring webHelpURL;
        self.webHelpURL = ''
        self._userPath = koprocessutils.getUserEnv()["PATH"].split(os.pathsep)


    # Query for a list of found installations of this application.
    # void FindInstallationPaths(out PRUint32 count,
    #                           [retval,
    #                            array,
    #                            size_is(count)] out wstring strs);
    def FindInstallationPaths(self):
        if sys.platform.startswith('win'):
            exts = ['.exe']
        else:
            exts = None
        self._executables = []
        installationPaths = None
        self._executables = which.whichall('ruby', exts=exts, path=self._userPath)
        if self._executables:
            installationPaths = [self.getInstallationPathFromBinary(p)\
                                   for p in self._executables]
        return installationPaths
    
    # Given the path to an interpreter binary, return the
    # path of the installation
    # wstring getInstallationPathFromBinary(in wstring binaryPath);
    def getInstallationPathFromBinary(self, binaryPath):
        if sys.platform.startswith("win"):
            return os.path.dirname(binaryPath)
        else:
            return os.path.dirname(os.path.dirname(binaryPath))
    
    def set_installationPath(self, path):
        self.installationPath = path
        self.executablePath = ''
    
    def get_executablePath(self):
        if not self.installationPath:
            if not self._executables:
                paths = self.FindInstallationPaths()
                if paths:
                    self.installationPath = paths[0]
            if not self._executables:
                return None
            self.executablePath = self._executables[0]
        return self.executablePath

class KoRubyCompileLinter:
    _com_interfaces_ = [components.interfaces.koILinter]
    _reg_desc_ = "Komodo Ruby Linter"
    _reg_clsid_ = "{9efff282-9919-4575-8c89-4a1e57512c97}"
    _reg_contractid_ = "@activestate.com/koRubyLinter;1"

    def __init__(self):
        self.appInfoEx = components.classes["@activestate.com/koRubyInfoEx;1"].\
                    getService(components.interfaces.koIAppInfoEx)

        self._lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"].\
            getService(components.interfaces.koILastErrorService)
        self._proxyMgr = components.classes["@mozilla.org/xpcomproxy;1"].\
            getService(components.interfaces.nsIProxyObjectManager)
        self._prefSvc = components.classes["@activestate.com/koPrefService;1"].\
            getService(components.interfaces.koIPrefService)
        self._prefProxy = self._proxyMgr.getProxyForObject(None,
            components.interfaces.koIPrefService, self._prefSvc,
            PROXY_ALWAYS | PROXY_SYNC)
        
    def lint(self, request):
        """Lint the given Ruby file.
        
        Raise an exception and set an error on koLastErrorService if there
        is a problem.
        """
        
        text = request.content.encode(request.encoding.python_encoding_name)
        cwd = request.cwd
        
        #print "----------------------------"
        #print "Ruby Lint"
        #print text
        #print "----------------------------"
        ruby = self.appInfoEx.executablePath
        if ruby is None:
            errmsg = "Could not find a suitable Ruby interpreter for linting."
            self._lastErrorSvc.setLastError(1, errmsg)
            raise ServerException(nsError.NS_ERROR_NOT_AVAILABLE)

        # save buffer to a temporary file
        try:
            filename = tempfile.mktemp()
            fout = open(filename, 'wb')
            fout.write(text)
            fout.close()
        except:
            errmsg = "Unable to save temporary file for Ruby linting."
            self._lastErrorSvc.setLastError(1, errmsg)
            raise ServerException(nsError.NS_ERROR_NOT_AVAILABLE)

        argv = [ruby, '-c', filename]
        _re = re.compile(r'(?P<file>[^:]+)?:(?P<line>\d+?):\s(?P<message>.*)')
        p = None
        #print argv
        try:
            env = koprocessutils.getUserEnv()
            cwd = cwd or None
            p = process.ProcessOpen(argv, mode='b', cwd=cwd, env=env)
            p.stdin.close()
            lines = p.stdout.readlines() + p.stderr.readlines()
            #print lines
        finally:
            os.unlink(filename)
            try:
                if p: p.close()
            except IOError, e:
                if e.errno == 0:  
                    pass
                else:
                    raise
        
        try:
            results = koLintResults()
            if lines[0] == "Syntax OK":
                return results
            if lines:
                datalines = re.split('\r\n|\r|\n',text)
                numLines = len(datalines)
                result = None
                for line in lines:
                    #print line
                    g = _re.match(line)
                    if not g:
                        continue
                    err = g.groupdict()
                    #print repr(err)
                    result = KoLintResult()
                    # XXX error in FILENAME at line XXX
                    result.lineStart = result.lineEnd = int(err['line'])
                    result.columnStart = 1
                    result.columnEnd = len(datalines[result.lineEnd-1]) + 1
                    result.severity = result.SEV_ERROR
                    result.description = err['message']
                    results.addResult(result)
        except:
            errmsg = "Exception in Ruby linting while parsing results"
            self._lastErrorSvc.setLastError(1, errmsg)
            log.exception(errmsg)
        #print "----------------------------"
        return results

