from xpcom import components, _xpcom
from koLanguageServiceBase import KoCompletionLanguageService
import string
import time
import re
from koXMLStateMachineReader import BasicNamespace
import xmllex
import sys
import logging

log = logging.getLogger("koXMLCompletionBase")
wordbreaks = ":-_" + string.uppercase + string.lowercase + string.digits


def uniqify(list):
    dict = {}
    for item in list:
        dict[item]=None
    return dict.keys()

class koXMLCompletionBase(KoCompletionLanguageService):
    namespace_handlers = {}

    def AutoComplete(self, ch, scimoz):
        #scitext = scimoz.text
        selStart, selEnd = scimoz.selectionStart, scimoz.selectionEnd

        style = scimoz.getStyleAt(scimoz.positionBefore(scimoz.currentPos)) & 127
        #if ch!=scitext[scimoz.currentPos-1]:
        #    log.warn("ch (%s) is not scitext[...](%s)" % (ch, scitext[scimoz.currentPos-1]))
        #print "keyPressed style: ",style," char: ", scitext[scimoz.currentPos-1]

        if style == scimoz.SCE_XML_START_TAG_NAME:
            pass
            # XXX the code below was too slow...so now we only trigger on
            # "<" sign...
            #if ch == ":":
            #     self._StartLocalNameAutoComplete(scimoz, scitext)
            #else:
            #    self._StartTagNameAutoComplete(scimoz, scitext)

        # until this is better worked out, dissable it.
        #elif style == scimoz.SCE_XML_DEFAULT and prev_style == scimoz.SCE_XML_START_TAG_NAME:
        #    self._FillinRequiredAttributes(scimoz, scitext)
        
        elif ch == "<" and style in [
            scimoz.SCE_XML_START_TAG_OPEN,
            scimoz.SCE_XML_DEFAULT,
            scimoz.SCE_XML_DATA_CHARS,
            scimoz.SCE_XML_DATA_NEWLINE]:
                self._StartTagNameAutoComplete(scimoz, scimoz.text)

        elif style in [getattr(scimoz,"SCE_XML_START_TAG_WHITE_SPACE", None),
                                    scimoz.SCE_XML_START_TAG_ATTR_NAME] or \
                (style == scimoz.SCE_XML_DEFAULT and ch in string.whitespace):
                # the last  is because of buggy lexers
                # this function already refused to call autoCShow if autoCActive, so avoid processing
            if not scimoz.autoCActive(): self._StartAttrAutoComplete(scimoz, scimoz.text)

        elif style == scimoz.SCE_XML_START_TAG_ATTR_QUOT_OPEN \
             or style == scimoz.SCE_XML_START_TAG_ATTR_APOS_OPEN:
            self._StartSuggestAutoComplete(scimoz, scimoz.text)

        elif (style == scimoz.SCE_XML_START_TAG_ATTR_QUOT_CLOSE \
               or style == scimoz.SCE_XML_START_TAG_ATTR_APOS_CLOSE \
           or style == scimoz.SCE_XML_START_TAG_ATTR_EQUALS or ch == ">" ) \
           and scimoz.autoCActive():
            scimoz.autoCCancel()
            
        else:
            chPos = scimoz.positionAtChar(scimoz.currentPos,-2)
            prev_style = scimoz.getStyleAt(chPos) & 127
            lastchar = scimoz.getWCharAt(chPos)
            if style == scimoz.SCE_XML_START_TAG_CLOSE and \
                ( # self._tagname or
                 prev_style == scimoz.SCE_XML_START_TAG_NAME) \
                and lastchar <> "/":
                if scimoz.autoCActive(): scimoz.autoCCancel()
                #self._tagname = None

            # not getting reliable styles for end-tags from lexer so ignore them
            elif ch=="/" and lastchar == "<":
                self._EndTagAutoComplete(scimoz, scimoz.text)
    
    def StartCallTip(self, ch, scimoz):
        pass

    def _StartTagNameAutoComplete(self, scimoz, scitext):
        tagnames = []
        curCharPos = scimoz.charPosAtPosition(scimoz.currentPos)
        lastpos = scitext.rfind("<", 0, curCharPos) - 1
        lastpos = max(lastpos, 0)
        if not lastpos:
            return
        assert scitext[lastpos+1]=="<"
        tree = xmllex.fastxmltree(scitext[:lastpos])
        if tree.root:
            controlling_namespace = xmllex.namespace(tree.root)

            handlerclass = get_namespace_handler(controlling_namespace,
                              scimoz, scitext, scimoz.positionBefore(scimoz.currentPos), tree)

            tagnames = handlerclass.tagnames()
            tagnames.sort()
          
            if tagnames and not scimoz.autoCActive():
                # XXX The '\n' literal should really be taken from
                #     self.completionSeparator from koLanguageServiceBase
                completions = '\n'.join(tagnames)
                diff = scimoz.positionAtChar(scimoz.currentPos, 0 - lastpos - 2)
                scimoz.autoCShow(diff, completions)

    def _StartLocalNameAutoComplete(self, scimoz, scitext):
        return #XXX This code just doesn't want to to work right!
        scimoz.autoCCancelAtStart = 0
        completions = None
        curCharPos = scimoz.charPosAtPosition(scimoz.currentPos)
        tree = xmllex.fastxmltree(scitext[:curCharPos])

        if not tree.current: return
        
        prefix = xmllex.prefix(tree.current)

        if prefix:
            parentTagName = xmllex.tagname(xmllex.parent(tree.current))
            
            controlling_namespace = xmllex.namespace(tree.root)
            handlerclass = get_namespace_handler(controlling_namespace,
                              scimoz, scitext, scimoz.positionBefore(scimoz.currentPos), tree)

            localnames = handlerclass.localnames(prefix)

            localnames.sort()

            # XXX The '\n' literal should really be taken from
            #     self.completionSeparator from koLanguageServiceBase
            completions = "\n".join(localnames)
            if completions:
#                if not scimoz.autoCActive():
                    scimoz.autoCShow(len(prefix)+1, completions)


    def getTagName(self, scimoz, scitext):
        text = _GetWordStop("<", scimoz.positionBefore(scimoz.selectionStart), scimoz, scitext)
        w = string.split(text,":")
        if len(w) < 1: return text,"",""
        if len(w) < 2: return text,w[0],""
        #self._tagname = w[1]
        return text,w[0],w[1]

    _cachedscimoz = None
        
    def _EndTagAutoComplete(self, scimoz, scitext):
        charPos = scimoz.positionAtChar(scimoz.currentPos,-2)
        tree = xmllex.fastxmltree(scitext[:charPos])
        if tree.current:        
            parentTagName = xmllex.tagname(tree.current)
#        pos, parentTagName, children = findParent(scimoz.currentPos-3, 
#                                                            scimoz, scitext, lambda : 0 )
            scimoz.autoCCancelAtStart = 0
            completions = parentTagName+">"

            if not scimoz.autoCActive():
                scimoz.autoCShow(0, completions)

    def _StartSuggestAutoComplete(self, scimoz, scitext):
        scimoz.autoCCancelAtStart = 0
        completions = None
        pos = scimoz.charPosAtPosition(scimoz.currentPos)
        tree = xmllex.fastxmltree(scitext[:pos])
        if not tree.current:
            return
        
        el = xmllex.tagname(tree.current)
        if not el:
            log.warn("no el in _StartSuggestAutoComplete")
            return

        controlling_namespace = xmllex.namespace(tree.root)
        if not controlling_namespace:
#            log.warn("no controlling_namespace in _StartSuggestAutoComplete")
            return
        
        handlerclass = get_namespace_handler(controlling_namespace,
                          scimoz, scitext, scimoz.positionBefore(scimoz.currentPos), tree)
        an = self._GetWordStop(" ", scimoz.positionBefore(scimoz.selectionStart), scimoz, scitext)

        if not an:
            log.warn("no an in _StartSuggestAutoComplete")
            return

        while pos>0 and scitext[pos] not in ('"', "'"):        
            pos -= 1
        if pos:
            quote = scitext[pos]
        else:
            quote = '"'

        values = [value + quote for value in handlerclass.values(an)]
        values.sort()

        if values:
            # XXX The '\n' literal should really be taken from
            #     self.completionSeparator from koLanguageServiceBase
            completions = "\n".join(values)
            if not scimoz.autoCActive():
                if completions:
                    charPos = scimoz.positionAtChar(scimoz.currentPos,0 - pos - 1)
                    scimoz.autoCShow(charPos, completions)
##                else:
##                    scimoz.autoCCancel()

    def _StartAttrAutoComplete(self, scimoz, scitext):
        scimoz.autoCCancelAtStart = 0
        completions = None
        currentPos = scimoz.charPosAtPosition(scimoz.currentPos)
        ltpos = scitext.find("<", currentPos)
        if ltpos==-1:
            ltpos = sys.maxint
            
        gtpos = scitext.find(">", currentPos)
        if gtpos==-1:
            gtpos = sys.maxint

        if gtpos==ltpos==sys.maxint:
            tagendpos = currentPos
        else:            
            tagendpos = min([ltpos, gtpos])

        tree = xmllex.fastxmltree(scitext[:tagendpos])

        if not tree.current: return
        
        el = xmllex.tagname(tree.current)
        controlling_namespace = xmllex.namespace(tree.root)

        handlerclass = get_namespace_handler(controlling_namespace,
                          scimoz, scitext, scimoz.positionBefore(scimoz.currentPos), tree)

        already_supplied = [name for name,value in xmllex.attrs(tree.current)]
    
        attrs = [name+"=" for name in handlerclass.attrs()
                     if name not in already_supplied]
        attrs.sort()

        an = self._GetWordStop(" ", scimoz.positionBefore(scimoz.selectionStart), scimoz, scitext)

        if el and attrs:
            # XXX The '\n' literal should really be taken from
            #     self.completionSeparator from koLanguageServiceBase
            completions = "\n".join([attr for attr in attrs if attr.startswith(an)])

            if completions:
                if not scimoz.autoCActive() :

                    scimoz.autoCShow(len(an), completions)
                    # XXX This seems to cause the completion menu not to appear.
                    # Someone who understands this code should investigate this.
                    #s.autoCSelect(word)
##                else:
##                    scimoz.autoCCancel()

   

    def _GetWordStop(self, stopchars, index, scimoz, scitext):
        return _GetWordStop(stopchars, index, scimoz, scitext)


def _GetWordStop(stopchars, byteIndex, scimoz, scitext):
    index = scimoz.charPosAtPosition(byteIndex)
    limit = len(scitext)
    index = min(index, limit-1)
    word = []

    # find the start
    while 0 <= index:
        char = scitext[index]
        if char in stopchars: break
        index = index - 1

    index = index + 1
    while index < limit:
        char = scitext[index]
        if char not in wordbreaks: break
        word.append(char)
        index = index + 1

    return string.join(word, '')

def stylename(scimoz, style):
   return " or ".join([i for i in dir(scimoz) if getattr(scimoz,i)==style
                                and i.startswith("SCE_XML")])

class koNamespaceHandler:
    handlers = {}
    def __init__(self, scimoz, scitext, pos, tree):
        self.scimoz = scimoz
        self.scitext = scitext
        self.pos = pos
        self.tree = tree
    
    def tagnames(self):
        return []

    def localnames(self):
        return []

    def attrs(self):
        return []

    def values(self, attrname):
        return []

class koDumbXMLNamespaceHandler(koNamespaceHandler):
    def __init__(self, scimoz, scitext, pos, tree):
        self.othertags = xmllex.fastxmltree(scitext[pos:]).tags
        koNamespaceHandler.__init__(self, scimoz, scitext, pos, tree)

    def tagnames(self):
        tags = [tag 
            for tag in self.othertags
                if not tag in self.tree.tags]
        return tags + self.tree.tags

    def localnames(self, prefix):
        return [name.split(":")[1] for name in
                        self.tree.tags if name.startswith(prefix+":")]

def get_namespace_handler(namespace, scimoz, scitext, pos, tree):
    handler = koNamespaceHandler.handlers.get(namespace, koDumbXMLNamespaceHandler)
    return handler(scimoz, scitext, pos, tree)

junk = """
tagcollectionre = re.compile(r"<([\w\-_]+(?::[\w\-_]+)?)",
                       re.UNICODE|re.DOTALL)
def collect(self, scitext, target):
    matches = tagcollectionre.findall(scitext)
    
    if matches:
        for match in matches:
            target[match]=None
"""
